// Arduino_ROV_Joystick_Nuts_Volts
// 
// Written by Theron Wierenga, January 2016
//
// This program reads the two joysticks, and maps the results to PWM numbers 0-255
// and then sends these numbers to the ROV
// It can receive battery data from the ROV and forward it to the PC
// It can also read two switches on the joysticks, these are not implimented
// 
// Defined commands sent to ROV
//
// LMF255 - left motor forward and PWM value
// LMR255 - left motor reverse and PWM value
// RMF255 - right motor forward and PWM value
// RMR255 - right motor reverse and PWM value
// UMU255 - up/down motor up and PWM value
// UMD255 - u/down motor down and PWM value
//
// Defined data sent back to joystick
// BAT followed by battery voltage as a float
// CMP followed by 359, compass bearing going clockwise
//
//
// Joystick Output
//
//        0
//
//
// 0              1023
//
//
//       1023
//
//
// Right Joystick
//
// X = A3     Left/Right movement
// Y = A2     Forward/Reverse movement
// Swirch = D3
//
// Left Joystick
//
// X = A6     Up/Down movement, has R to ground to make audio taper linear, used 4.7k
// Y = A7     Not used
// Switch = D2
//
// Battery level is read on A0, not used
//
// Serial to ROV Arduino is D10, D11, RX, TX
//

#include <SoftwareSerial.h>
SoftwareSerial mySerial(10, 11); // RX, TX
String inputString = "", strTemp = "";
boolean stringComplete = false;
int xRight, xLeft, yRight, yLeft, xTemp, yTemp, iR, iL, lastiL = 0, lastiR = 0, lastxLeft = 0;
float xNorm, yNorm, R, L, nx;

void setup() 
{
  Serial.begin(9600);
  
  // reserve 200 bytes for the inputString:
  inputString.reserve(200);
  mySerial.begin(9600);
}

void loop() 
{
    // Read joysticks - returns 0 to 1023
    
    xRight = analogRead(A3);
    yRight = analogRead(A2); 
    xLeft = analogRead(A6);
    yLeft = analogRead(A7);

//    Serial.print(xLeft);
//    Serial.print("    ");
//    Serial.print(yLeft);
//    Serial.print("    ");
//    Serial.print(xRight);
//    Serial.print("    ");
//    Serial.print(yRight);
//    Serial.println("    ");

    // Normalize - returns a float -1 to +1 for x, + 1 to - 1 for y

    xNorm = normalizeX(float(xRight));
    yNorm = normalizeY(float(yRight));

    // Now apply wheelchair algorithm
    
    if ((xNorm / 2.0) > 0.1)
    {
        nx = 0.1 + (xNorm /2.0 - 0.1) / 2.0;
    }
    else if ((xNorm / 2.0) < -0.1)
    {
        nx = -0.1 + (xNorm /2.0 + 0.1) / 2.0;
    }
    else
    {
        nx = xNorm/2.0;
    }

    R = yNorm - nx;
    L = yNorm + nx;

    // Clip as nessary
    
    if (R > 1.0) R = 1.0;
    if (R < -1.0) R = -1.0;
    if ((R <= 0.05) && (R >= -0.05)) R = 0.0;
    if (L > 1.0) L = 1.0;
    if (L < -1.0) L = -1.0;
    if ((L <= 0.05) && (L >= -0.05)) L = 0.0;

    iL = int(L * 255.0);
    iR = int(R * 255.0);

//    Serial.print("L and R   ");
//    Serial.print(iL);
//    Serial.print("  ");
//    Serial.println(iR);

    if (iL != lastiL)
    {
        strTemp = formatString(abs(iL));
      //  Serial.print("L = ");
      //  Serial.println(strTemp);
        if (iL > 0)
        { 
            mySerial.println("LMF" + strTemp);
        }
        else
        {
            mySerial.println("LMR" + strTemp);
        }
        lastiL = iL;

     //   Serial.println("iL = " + strTemp);
    }
   

    if (iR != lastiR)
    {
        strTemp = formatString(abs(iR));
        if (iR > 0)
        { 
            mySerial.println("RMF" + strTemp);
        }
        else
        {
            mySerial.println("RMR" + strTemp);
        }
        lastiR = iR;
        
      //  Serial.println("iR = " + strTemp);
    }

     
     
    // Now look at Up/Down - strictly linear

    if (xLeft != lastxLeft)
    {
        //  Serial.print("U/D = ");
        //  Serial.println(xLeft);
        // Up
        if (xLeft < 512)    // Using X because of pin configuration on slider pot
        {
            yTemp = 255 - (xLeft / 2);
            if (yTemp < 15) yTemp = 0;
            strTemp = formatString(yTemp);
            mySerial.println("UMU" + strTemp);
        }
        // Down
        else
        {
            yTemp = 255 - ((1023 - xLeft) / 2);
            if (yTemp < 15) yTemp = 0;
            strTemp = formatString(yTemp);
            mySerial.println("UMD" + strTemp);
        }
        lastxLeft = xLeft;

    }

    // Now check for any messages received from the ROV controller
    
    mySerialEvent();
    if (stringComplete)
    {
        String subStr = inputString.substring(0, 3);
        // Battery voltage
        if (subStr == "BAT")
        {
            String temp = inputString.substring(3, inputString.length());
            float val = temp.toFloat();
            Serial.print("ROV battery voltage = ");
            Serial.println(val);
        }
        // Compass heading
        if (subStr == "CMP")
        {
            String temp = inputString.substring(3, 6);
            byte val = (byte)temp.toInt(); 
            Serial.print("Compass heading = ");
            Serial.println(val);
        }
        stringComplete = false;
        inputString = "";
    }
    delay(200);
}

// For receiving characters

void mySerialEvent() 
{
    while ((mySerial.available() > 0) && (stringComplete == false))
    {
        // get the new byte:
        char inChar = (char)mySerial.read();
        // add it to the inputString:
        inputString += inChar;
        delay(2);                                                 // do i need this?
        // if the incoming character is a newline, set a flag
        // so the main loop can do something about it:
        if (inChar == '\n') 
        {
          stringComplete = true;
        }
    }
}

// Adds leading 0's as necessary
String formatString(int temp)
{
    // Use int to create 3 digit string with leading 0's if necessary
     String sTemp = String(temp);
     while (sTemp.length() < 3)
     {
        sTemp = "0" + sTemp;
     }
     return sTemp;
}

// Takes in 0 to 1023
// Returns -1 to +1

float normalizeX(float x)
{
    return ((x / 1023.0) * 2.0) - 1.0;
}

// Takes in 0 to 1023
// Returns +1 to -1

float normalizeY(float x)
{
    return (((1023.0 - x) / 1023.0) * 2.0) - 1.0;
}







